'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /**
                                                                                                                                                                                                                                                                   * The top-level merge function
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * @param {object} remerge map
                                                                                                                                                                                                                                                                   * @return {function} reducer function
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   */

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var merge = function merge(map) {

  var getAccessorKey = function getAccessorKey(key) {
    // this regex tests if the key is of the form abc[123], with opening and closing square brackets
    var containsAccessor = /\w+\[\w+\]$/.test(key);
    if (containsAccessor) {
      // this regex captures the content inside the square brackets
      var captureAccessor = /.+\[(\w+)\]/.exec(key);
      if (captureAccessor) {
        return captureAccessor[1];
      } else {
        return null;
      }
    } else {
      return null;
    }
  };

  var removeAccessorKey = function removeAccessorKey(key) {
    if (getAccessorKey(key)) {
      return key.replace(getAccessorKey(key), "");
    }
    return key;
  };

  var _preprocess = function _preprocess(_originalMap) {
    var _map = _lodash2.default.cloneDeep(_originalMap);
    var newMap = {};

    for (var key in _originalMap) {
      var isFunction = _lodash2.default.isFunction(_map[key]);
      newMap[removeAccessorKey(key)] = {
        key: /([^\[]+)/.exec(key)[1],
        isLeaf: isFunction,
        accessorKeyName: getAccessorKey(key),
        child: isFunction ? _map[key] : _preprocess(_map[key])
      };
    }

    return newMap;
  };

  var _process = function _process(_map, state, action) {

    var currentPath = action.type.split('.', 1)[0];

    var newState = _lodash2.default.cloneDeep(state);

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = Object.keys(_map)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var path = _step.value;

        if (path === currentPath) {
          var _map$path = _map[path];
          var key = _map$path.key;
          var accessorKeyName = _map$path.accessorKeyName;
          var isLeaf = _map$path.isLeaf;
          var child = _map$path.child;


          if (isLeaf) {

            return child(newState, action);
          } else {

            var smallerMap = child;
            var smallerState = accessorKeyName ? newState[key][action[accessorKeyName]] : newState[key];
            var smallerAction = _extends({}, action, {
              type: action.type.split('.').splice(1).join(".")
            });
            var newSmallerState = _process(smallerMap, smallerState, smallerAction);

            if (accessorKeyName) {
              newState[key][action[accessorKeyName]] = newSmallerState;
            } else {
              newState[key] = newSmallerState;
            }
          }
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return newState;
  };

  var computedMap = _preprocess(map);

  return function (state, action) {
    return _process(computedMap, state, action);
  };
};

exports.default = merge;
